/* 
   HTTP utility functions
   Copyright (C) 1999, Joe Orton <joe@orton.demon.co.uk>
                                                                     
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
  
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef HTTP_UTILS_H
#define HTTP_UTILS_H

#include <config.h>

#include <sys/types.h>

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#ifdef HAVE_STDARG_H
#include <stdarg.h>
#endif

#ifdef HAVE_DEBUG_FUNC
#include "common.h"
#endif

#define HTTP_QUOTES "\"'"
#define HTTP_WHITESPACE " \r\n\t"

/* Handy macro to free things. */
#define HTTP_FREE(x) { if( (x)!=NULL ) free( x ); }

#define EOL "\r\n"
#define HTTP_PORT 80

time_t http_dateparse( const char *date );

#undef min
#define min(a,b) ((a)<(b)?(a):(b))

#ifndef HAVE_DEBUG_FUNC
/* CONSIDER: mutt has a nicer way of way of doing debugging output... maybe
 * switch to like that. */
#ifdef DEBUGGING
#define DEBUG if(0) http_debug
#else /* !DEBUGGING */
#define DEBUG http_debug
#endif /* DEBUGGING */

void http_debug( int ch, char *, ... )
#ifdef __GNUC__
                __attribute__ ((format (printf, 2, 3)))
#endif /* __GNUC__ */
;

#endif /* HAVE_DEBUG_FUNC */

/* String buffer handling */

struct http_buffer {
    char *data;
    size_t used; /* used bytes in buffer */
    size_t length; /* length of buffer */
};

static inline int http_buffer_grow( struct http_buffer *buf, size_t newsize );

/* Grows for given size, 0 on success, -1 on error */
static inline int http_buffer_grow( struct http_buffer *buf, size_t newsize ) {
    size_t newlen, oldbuflen;

#define HTTP_BUFFER_GROWTH 512

    if( newsize <= buf->length ) return 0; /* big enough already */
    /* FIXME: ah, can't remember my maths... better way to do this? */
    newlen = ( (newsize / HTTP_BUFFER_GROWTH) + 1) * HTTP_BUFFER_GROWTH;
    
    oldbuflen = buf->length;
    /* Reallocate bigger buffer */
    buf->data = realloc( buf->data, newlen );
    if( buf->data == NULL ) return -1;
    buf->length = newlen;
    /* Zero-out the new bit of buffer */
    memset( buf->data+oldbuflen, 0, newlen-oldbuflen );

    return 0;
}

/* Concatenate all given strings onto the end of the buffer.
 * The strings must be null-terminated, and must be followed by a
 * NULL argument marking the end of the list.
 */
int http_buffer_concat( struct http_buffer *buf, ... );

struct http_buffer *http_buffer_create( void );
void http_buffer_destroy( struct http_buffer *buf );

/* Append zero-terminated string */
int http_buffer_zappend( struct http_buffer *buf, const char *str );

/* Append number of bytes from data */
int http_buffer_append( struct http_buffer *buf, const char *data, size_t len );

inline static void http_buffer_clear( struct http_buffer *buf );

inline static void http_buffer_clear( struct http_buffer *buf ) {
    memset( buf->data, 0, buf->length );
    buf->used = 0;
}  

#endif /* HTTP_UTILS_H */
